@@ -2,13 +2,13 @@ use crate::{
     searchresult::SearchResult,
     structs::{Buffers, FileType, Output, Search},
 };
-use rayon::iter::{ParallelBridge, ParallelIterator};
 use std::path::Path;
 
 type Receiver = crossbeam_channel::Receiver<SearchResult>;
 type Sender = crossbeam_channel::Sender<SearchResult>;
 
 impl Search {
+    #[profi::profile]
     pub fn search(&self) -> Buffers {
         let (sender, receiver) = crossbeam_channel::bounded(8);
 
@@ -43,67 +43,72 @@ impl Search {
 
         // Search in directories
         rayon::scope(move |s| {
-            s.spawn(move |_| {
-                dirs.par_bridge()
-                    .for_each(|dir| search_dir(dir.as_ref(), self, sender.clone()))
-            });
+            for dir in dirs {
+                let sender = sender.clone();
+                s.spawn(move |_| search_dir(dir.as_ref(), self, sender));
+            }
+            drop(sender);
             receive_paths(receiver, self)
         })
     }
 }
 
+#[profi::profile]
 fn search_dir(path: impl AsRef<Path>, search: &Search, sender: Sender) {
     let path = path.as_ref();
-    let Ok(read) = std::fs::read_dir(path) else {
-        if search.verbose {
-            eprintln!("Could not read {:?}", path);
-        }
-        return;
+    
+    let read ={
+        profi::prof!("search_dir::read_dir");
+        let Ok(read) = std::fs::read_dir(path) else {
+            if search.verbose {
+                eprintln!("Could not read {:?}", path);
+            }
+            return;
+        };
+        read
     };
 
     rayon::scope(|s| {
+        profi::prof!("search_dir::inspect_entries");
         for entry in read.flatten() {
+            profi::prof!("search_dir::inspect_entry");
             let Some((result, is_dir)) = is_result(entry, search) else {
                 continue;
             };
             if let Some(result) = result {
+                profi::prof!("search_dir::send_result");
                 sender.send(result).unwrap();
             }
             if let Some(path) = is_dir {
+                profi::prof!("search_dir::spawn_search_dir");
                 s.spawn(|_| search_dir(path, search, sender.clone()));
             }
         }
-    })
+    });
 }
 
+#[profi::profile]
 fn is_result(
     entry: std::fs::DirEntry,
     search: &Search,
 ) -> Option<(Option<SearchResult>, Option<Box<Path>>)> {
     // Get entry name
-    let path = entry.path();
-    let fname = file_name(&path).unwrap();
-    let fname = fname.to_string_lossy();
-    let sname: std::borrow::Cow<str> = if search.case_sensitive {
-        fname.as_ref().into()
-    } else {
-        fname.to_ascii_lowercase().into()
+    let path = {
+        profi::prof!("is_result::entry.path");
+        entry.path()
     };
 
-    if search.explicit_ignore.binary_search(&path).is_ok() {
-        return None;
+    {
+        profi::prof!("is_result::explicit_ignore");
+        if search.explicit_ignore.binary_search(&path).is_ok() {
+            return None;
+        }
     }
-
-    let hardcoded = || {
-        search
-            .hardcoded_ignore
-            .binary_search_by(|p| std::path::Path::new(p).cmp(&path))
-            .is_ok()
-    };
-
+    
     let is_hidden = || {
         #[cfg(unix)]
         {
+            profi::prof!("is_result::is_hidden");
             is_hidden(&path)
         }
         #[cfg(windows)]
@@ -112,42 +117,85 @@ fn is_result(
         }
     };
 
-    if !search.hidden && (is_hidden() || hardcoded()) {
-        return None;
+    {
+        profi::prof!("is_result::hidden_check");
+        if !search.hidden && is_hidden() {
+            return None;
+        }
     }
 
     // Read type of file and check if it should be added to search results
-    let is_dir = matches!(entry.file_type(), Ok(ftype) if ftype.is_dir());
-    let ftype = match search.ftype {
-        FileType::Dir => is_dir,
-        FileType::File => !is_dir,
-        FileType::All => true,
+    let is_dir = {
+        profi::prof!("is_result::is_dir");
+        matches!(entry.file_type(), Ok(ftype) if ftype.is_dir())
+    };
+    let ftype = {
+        profi::prof!("is_result::get_ftype");
+        match search.ftype {
+            FileType::Dir => is_dir,
+            FileType::File => !is_dir,
+            FileType::All => true,
+        }
+    };
+    
+    let Some(fname) = file_name(&path) else {
+        profi::prof!("is_result::return_invalid_file_name");
+        return Some((None, is_dir.then_some(path.into_boxed_path())))
+    };
+    let fname = {
+        profi::prof!("is_result::fname.to_string_lossy");
+        fname.to_string_lossy()
+    };
+    let sname: std::borrow::Cow<str> = if search.case_sensitive {
+        profi::prof!("is_result::sname");
+        fname.as_ref().into()
+    } else {
+        profi::prof!("is_result::to_ascii_lowercase");
+        fname.to_ascii_lowercase().into()
     };
 
-    let starts = search.starts.is_empty() || sname.starts_with(&search.starts);
-    let ends = search.ends.is_empty() || sname.ends_with(&search.ends);
-
-    if starts && ends && ftype {
+    let starts = || {
+        profi::prof!("is_result::starts_with");
+        search.starts.is_empty() || sname.starts_with(&search.starts)
+    };
+    let ends = || {
+        profi::prof!("is_result::ends_with");
+        search.ends.is_empty() || sname.ends_with(&search.ends)
+    };
+    
+    profi::prof!("is_result::substring_checks");
+    if ftype && starts() && ends() {
+        let (equals, contains) = {
+            profi::prof!("is_result::contains");
+            if search.finder.find(sname.as_bytes()).is_none() {
+                (false, false)
+            } else {
+                (sname.len() == search.name.len(), true)
+            }
+        };
         // If file name is equal to search name, write it to the "Exact" buffer
-        if sname == search.name {
+        if equals {
+            profi::prof!("is_result::return_exact");
             return Some((
                 Some(SearchResult::exact(path.to_string_lossy().into_owned())),
                 is_dir.then_some(path.into_boxed_path()),
             ));
         }
         // If file name contains the search name, write it to the "Contains" buffer
-        else if !search.exact && sname.contains(&search.name) {
+        else if !search.exact && contains {
             let s = if search.output == Output::Normal {
                 crate::print::format_with_highlight(&fname, &sname, &path, search)
             } else {
                 path.to_string_lossy().into_owned()
             };
+            profi::prof!("is_result::return_contains");
             return Some((
                 Some(SearchResult::contains(s)),
                 is_dir.then_some(path.into_boxed_path()),
             ));
         }
     }
+    profi::prof!("is_result::return_not_found");
     Some((None, is_dir.then_some(path.into_boxed_path())))
 }
 
@@ -239,6 +287,7 @@ pub(crate) fn is_hidden(entry: &std::fs::DirEntry) -> bool {
 ///
 /// If the path terminates in ., .., or consists solely of a root of prefix,
 /// file_name will return None.
+#[profi::profile]
 #[cfg(unix)]
 #[inline(always)]
 pub(crate) fn file_name<P: AsRef<Path> + ?Sized>(path: &P) -> Option<&std::ffi::OsStr> {
